# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'expert_dlg.ui'
##
## Created by: Qt User Interface Compiler version 6.8.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractButton, QApplication, QDialog, QDialogButtonBox,
    QFrame, QSizePolicy, QTextBrowser, QWidget)

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        if not Dialog.objectName():
            Dialog.setObjectName(u"Dialog")
        Dialog.resize(581, 392)
        Dialog.setModal(True)
        self.buttonBox = QDialogButtonBox(Dialog)
        self.buttonBox.setObjectName(u"buttonBox")
        self.buttonBox.setGeometry(QRect(230, 350, 341, 32))
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Ok)
        self.textBrowser = QTextBrowser(Dialog)
        self.textBrowser.setObjectName(u"textBrowser")
        self.textBrowser.setGeometry(QRect(10, 10, 561, 331))
        self.textBrowser.setInputMethodHints(Qt.ImhNone)
        self.textBrowser.setFrameShadow(QFrame.Sunken)
        self.textBrowser.setOpenExternalLinks(True)

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)

        QMetaObject.connectSlotsByName(Dialog)
    # setupUi

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QCoreApplication.translate("Dialog", u"About expert parameters", None))
        self.textBrowser.setHtml(QCoreApplication.translate("Dialog", u"<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><meta charset=\"utf-8\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"hr { height: 1px; border-width: 0; }\n"
"li.unchecked::marker { content: \"\\2610\"; }\n"
"li.checked::marker { content: \"\\2612\"; }\n"
"</style></head><body style=\" font-family:'Segoe UI'; font-size:9pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:11pt; font-weight:700;\">Are you sure you know what these do?</span></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Before modifying these, you should read the documentation and the references listed below for a deep understading of what the algorithm does.</p>\n"
"<p style=\" margin-top:12px"
                        "; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">References:</p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Cabo, C., Ordonez, C., Lopez-Sanchez, C. A., &amp; Armesto, J. (2018). Automatic dendrometry: Tree detection, tree height and diameter estimation using terrestrial laser scanning. International Journal of Applied Earth Observation and Geoinformation, 69, 164\u2013174. <a href=\"https://doi.org/10.1016/j.jag.2018.01.011\"><span style=\" text-decoration: underline; color:#0000ff;\">10.1016/j.jag.2018.01.011</span></a></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Ester, M., Kriegel, H.-P., Sander, J., &amp; Xu, X. (1996). A Density-Based Algorithm for Discovering Clusters in Large Spatial Databases with Noise. www.aaai.org</p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px"
                        "; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Prendes, C., Cabo, C., Ordo\u00f1ez, C., Majada, J., &amp; Canga, E. (2021). An algorithm for the automatic parametrization of wood volume equations from Terrestrial Laser Scanning point clouds: application in Pinus pinaster. GIScience and Remote Sensing, 58(7), 1130\u20131150. <a href=\"https://doi.org/10.1080/15481603.2021.1972712\"><span style=\" text-decoration: underline; color:#0000ff;\">10.1080/15481603.2021.1972712</span></a></p>\n"
"<p style=\" margin-top:12px; margin-bottom:12px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Zhang, W., Qi, J., Wan, P., Wang, H., Xie, D., Wang, X., &amp; Yan, G. (2016). An easy-to-use airborne LiDAR data filtering method based on cloth simulation. Remote Sensing, 8(6). <a href=\"10.3390/rs8060501\"><span style=\" text-decoration: underline; color:#0000ff;\">https://doi.org/10.3390/rs8060501</span></a></p></body></html>", None))
    # retranslateUi




__version__ = "0.6.0"

# Copyright infos
__copyright_info_1__ = "3DFin: Forest Inventory Copyright (C) 2023-2025 Carlos Cabo & Diego Laino."
__copyright_info_2__ = (
    "This program comes with ABSOLUTELY NO WARRANTY. This is a free "
    "software, and you are welcome to redistribute it under certain conditions."
)

__copyright_info_3__ = "See LICENSE at the bottom of this tab for further details."

__license_msg__ = "See License at the bottom of 'About' tab for more details or visit <https://www.gnu.org/licenses/>"


# About the project
__about_1__ = (
    "This software has been developed at the Centre of Wildfire Research of "
    "Swansea University (UK) in collaboration with the Research Institute of\n"
    "Biodiversity (CSIC, Spain) and the Department of Mining Exploitation of the "
    "University of Oviedo (Spain). Funding provided by the UK NERC\nproject "
    "(NE/T001194/1):"
)

__nerc_project__ = "'Advancing 3D Fuel Mapping for Wildfire Behaviour and Risk Mitigation Modelling'"
__csic_project__ = (
    "'Advancing carbon emission estimations from wildfires applying "
    "artificial intelligence to 3D terrestrial point clouds'"
)
__about_2__ = "and by the Spanish Knowledge Generation project (PID2021-126790NB-I00):"



import configparser
from pathlib import Path

from three_d_fin.processing.configuration import FinConfiguration


def test_load_from_file():
    """Test configuration consistency.

    Load a valid config file with a non-default parameter (lower_limit).
    Parse it as an object and check that the lower_limit field is not set to
    default value anymore.

    Additionally do an obvious check: a parsed config file is equivalent to the
    same file loaded via static method From_config_file().
    """
    config_file = configparser.ConfigParser()
    config_file.read(Path("tests/lowerlimitchange.ini").resolve())
    valid_config = FinConfiguration.parse_obj(config_file)

    assert valid_config.basic.lower_limit == config_file.getfloat("basic", "lower_limit")

    assert valid_config == FinConfiguration.From_config_file(Path("tests/lowerlimitchange.ini"))


def test_write_to_file():
    """Test configuration file writing."""
    config = FinConfiguration()
    config.to_config_file(Path("tests/configtest.ini"))



import argparse
import configparser
import os
import sys
from pathlib import Path

from pydantic import ValidationError


def launch_application() -> int:
    """Parse the command line and launch the GUI or the CLI application.

    Main entry point for the 3DFin application:
        - Launch the GUI if the command is called without aguments.

        - Launch the CLI by appending the 'cli' subcommand to the command. For other
          arguments, the reader should refers to the body of the function.

    Returns
    -------
    exit_code : int
        POSIX minimal exit code (0 = SUCCESS, 1 = ERROR)

    """
    import laspy
    import pydantic
    from PySide6 import QtCore
    from PySide6.QtWidgets import QApplication

    from three_d_fin import __about__
    from three_d_fin.gui.application import Application
    from three_d_fin.processing.configuration import FinConfiguration, MiscParameters
    from three_d_fin.processing.standalone_processing import StandaloneLASProcessing

    EXIT_ERROR = 1
    EXIT_SUCCESS = 0

    parser = argparse.ArgumentParser(
        prog="3DFin",
        description=f"""
            {__about__.__copyright_info_1__}

            {__about__.__copyright_info_2__}
            {__about__.__license_msg__}""",
    )
    parser.add_argument("--version", "-v", action="version", version=__about__.__version__)

    # Create a subparser for cli subcommand
    subparsers = parser.add_subparsers(dest="subcommand")
    cli_subparser = subparsers.add_parser("cli", help="launch the app in command line mode")
    cli_subparser.add_argument("input_file", help="Las or Laz input file")
    cli_subparser.add_argument("output_directory", help="output directory where to put the results")
    cli_subparser.add_argument("params_file", help=".ini files with parameters")
    cli_subparser.add_argument(
        "--export_txt",
        action="store_true",
        help="Export tabular data in ASCII (space separated) files instead of XLSX",
    )
    cli_subparser.add_argument("--normalize", action="store_true", help="Normalize the data with CSF algorithm")
    cli_subparser.add_argument(
        "--denoise",
        action="store_true",
        help="Denoise the data, if outliers below ground level are expected",
    )
    cli_subparser.add_argument("--version", "-v", action="version", version=__about__.__version__)

    cli_parse = parser.parse_args()

    print(__about__.__copyright_info_1__)
    print(__about__.__copyright_info_2__)
    print(__about__.__license_msg__)

    fin_processing = StandaloneLASProcessing(FinConfiguration())
    # No subcommand, launch the GUI
    if cli_parse.subcommand is None:
        # for legacy purpose we look for a configuration file on the cwd
        try:
            config_file_path = Path("3DFinconfig.ini")
            config = FinConfiguration.From_config_file(config_file_path.resolve(strict=True), init_misc=True)
            print("Configuration file found. Setting default parameters from the file")
            fin_processing.set_config(config)
        except ValidationError:
            # Error message in this case, but stick to default parameters
            print("Configuration file found but it has validation errors, fallback to default")
            pass
        except FileNotFoundError:
            # No error message in this case, stick to default parameters
            pass

        os.environ["QT_AUTO_SCREEN_SCALE_FACTOR"] = "1"
        QApplication.setAttribute(QtCore.Qt.AA_EnableHighDpiScaling)

        app = QApplication(sys.argv)
        app_widget = Application(fin_processing)
        app_widget.show()
        app.exec_()
        return EXIT_SUCCESS

    # Else, the CLI case
    # First, we read the param file and sanitize the input
    config_path = Path(cli_parse.params_file)
    if not config_path.exists() or not config_path.is_file():
        print("Parameters: File does not exist")
        return EXIT_ERROR

    try:
        valid_params = FinConfiguration.From_config_file(config_path)
    except pydantic.ValidationError as v:
        print(repr(v))  # TODO: minimal display for now
    except configparser.ParsingError:
        print("Parameters: invalid .ini file")
        return EXIT_ERROR
    except configparser.NoOptionError as error:
        print(f"Parameters: invalid option '{error.option}' in section '{error.section}'")
        return EXIT_ERROR
    except ValueError as error:
        print(f"Parameters: {error.args[0]}")
        return EXIT_ERROR

    # Second, we check the validity of the las file
    input_las = Path(cli_parse.input_file)
    if not input_las.exists() or not input_las.is_file():
        print("Input file: file does not exists")
        return EXIT_ERROR
    try:
        laspy.open(input_las, read_evlrs=False)
    except laspy.LaspyException:
        print("Input file: invalid las file")
        return EXIT_ERROR

    # At last, We check the validity of the current output directory
    output_dir = Path(cli_parse.output_directory)
    if (
        not output_dir.exists()
        or not output_dir.is_dir()
        or not os.access(
            output_dir, os.W_OK
        )  # os.access won't work very well on Windows, we may still have to mess with exceptions
    ):
        print("Invalid output directory")
        return EXIT_ERROR

    # Wrap all in the misc section
    misc = MiscParameters(
        is_normalized=not cli_parse.normalize,
        is_noisy=cli_parse.denoise,
        export_txt=cli_parse.export_txt,
        input_file=cli_parse.input_file,
        output_dir=cli_parse.output_directory,
    )
    # Seems akward but we do not to enforce mutability on parameter for now
    final_params: FinConfiguration = FinConfiguration(
        basic=valid_params.basic,
        advanced=valid_params.advanced,
        expert=valid_params.expert,
        misc=misc,
    )
    # Run processing
    fin_processing.set_config(final_params)
    fin_processing.process()
    return EXIT_SUCCESS
